@@ -408,7 +408,7 @@ gridlist[speciesindex]["distributiontype"] = GridObject
     deltas = [h5file.root.parameters_uv.grid_spacing.read()]*3
     origin = [-deltas[dim]*gridcounts[dim]*0.5 for dim in range(3)]
 
-    speciesnames = h5file.root.parameters_vv.solvent.element_0.names[:]
+    speciesnames = h5file.root.data_vv.parameters_vv.solvent.element_0.names[:]
     grids = dict((speciesname, {}) for speciesname in speciesnames)
     for key, value in h5file.root._v_children.iteritems():
         # 
